@@ -52,11 +52,11 @@ class ErrorLogView(grok.View):
 
     def update(self):
         if 'ignore_errors_update' in self.request.form:
-            reporter.ignore_errors = self.request.form.get(
+            reporter.ignored_errors = self.request.form.get(
                 'ignore_errors', [])
 
         self.all_errors = reporter.all_ignored_errors
-        self.ignored_errors = reporter.ignore_errors
+        self.ignored_errors = reporter.ignored_errors
         self.errors = reporter.get_last_errors()
         self.debug_mode = self.request.response.debug_mode
 
@@ -94,40 +94,12 @@ class ErrorReporter(object):
         'Unauthorized', 'Forbidden',
         'BadRequest', 'BrokenReferenceError']
 
-    def __init__(self):
+    def __init__(self, subscribed=[]):
         self.__last_errors = collections.deque([], 25)
-        self.__ignore_errors = self.all_ignored_errors
-
-    @apply
-    def ignore_errors():
-        def getter(self):
-            return self.__ignore_errors
-        def setter(self, value):
-            self.__ignore_errors = set(value).intersection(
-                set(self.all_ignored_errors))
-        return property(getter, setter)
-
-    def get_last_errors(self):
-        """Return all last errors.
-        """
-        errors = list(self.__last_errors)
-        errors.reverse()
-        return errors
-
-    def is_loggable(self, error):
-        """Tells you if this error is loggable.
-        """
-        error_name = error.__class__.__name__
-        return error_name not in self.__ignore_errors
-
-    def log_last_error(self, request, response, obj=None, extra=None):
-        """Build an error report and log the last available error.
-        """
-        error_type, error_value, traceback = sys.exc_info()
-        if ((not response.debug_mode) and
-            (not self.is_loggable(error_value))):
-            return
+        self.__ignore_errors = set(self.all_ignored_errors)
+        self.__subscribed = list(subscribed) + [self._subscriber]
 
+    def _subscriber(self, request, response, obj, exc_info, exc_text, extra):
         log_entry = ['\n']
 
         if extra is not None:
@@ -148,16 +120,70 @@ class ErrorReporter(object):
         log_request_info('User-agent', 'HTTP_USER_AGENT')
         log_request_info('Refer', 'HTTP_REFERER')
 
-        log_entry.extend(format_exception(error_type, error_value, traceback))
-        self.log_error(request['URL'], ''.join(log_entry))
+        log_entry.extend(exc_text)
 
-
-    def log_error(self, url, report):
-        """Log a given error.
-        """
+        # Save error.
+        report = ''.join(log_entry)
         logger.error(report)
         self.__last_errors.append(
-            {'url': url, 'report': report, 'time': datetime.now()})
+            {'url': request['URL'], 'report': report, 'time': datetime.now()})
+
+    def subscribe_to_errors(self, handler):
+        """Subscribe when an error happens.
+        """
+        self.__subscribed.append(handler)
+
+    def show_errors(self, errors):
+        """Show the given errors.
+        """
+        self.__ignore_errors -= set(errors)
+
+    def ignore_errors(self, errors):
+        """Ignore the given errors.
+        """
+        self.__ignore_errors += set(errors)
+
+    @apply
+    def ignored_errors():
+        """Access currently ignored errors.
+        """
+
+        def getter(self):
+            return list(self.__ignore_errors)
+
+        def setter(self, errors):
+            self.__ignore_errors = set(errors)
+
+        return property(getter, setter)
+
+    def is_loggable(self, error):
+        """Tells you if this error is loggable.
+        """
+        error_name = error.__class__.__name__
+        return error_name not in self.__ignore_errors
+
+    def log_last_error(self, request, response, obj=None, extra=None):
+        """Build an error report and log the last available error.
+        """
+        error_type, error_value, traceback = exc_info = sys.exc_info()
+        try:
+            if ((not response.debug_mode) and
+                (not self.is_loggable(error_value))):
+                return
+
+            exc_text = format_exception(error_type, error_value, traceback)
+
+            for plugin in self.__subscribed:
+                plugin(request, response, obj, exc_info, exc_text, extra)
+        finally:
+            del traceback
+
+    def get_last_errors(self):
+        """Return all last errors.
+        """
+        errors = list(self.__last_errors)
+        errors.reverse()
+        return errors
 
 
 reporter = ErrorReporter()
