@@ -9,6 +9,7 @@ import sys
 
 from Zope2 import startup
 from infrae.wsgi.publisher import WSGIApplication
+from infrae.wsgi.log import reporter
 from zope.event import notify
 from zope.processlifetime import ProcessStarting
 import App.config
@@ -102,16 +103,64 @@ def boot_zope(config_filename, debug_mode=False):
     finally:
         bootstrap_lock.release()
 
+def parse_raven_config(options):
+    config = configuration_dict(options, 'raven', sep=".")
+    for key in ['exclude_paths', 'include_paths', 'processors']:
+        if key in config:
+            config[key] = configuration_list(config[key])
+    if config:
+        from .ravenplugin import RavenLoggingPlugin
+        reporter.subscribe_to_errors(RavenLoggingPlugin(config))
 
 def zope2_application_factory(global_conf, zope_conf, **options):
     """Build a Zope2 WSGI application.
     """
+    parse_raven_config(options)
+
     debug_mode = options.get('debug_mode', 'off') == 'on'
-    zope_workers = int(options.get('zope_workers', '4'))
+    zope_workers = configuration_int(options, 'zope_workers')
     boot_zope(zope_conf, debug_mode)
 
+    # By default debug mode does not render exceptions as error pages
+    # but allow them to propagate to outer wsgi middlewares
+    # (e.g. z3c.evalexception). Setting this to 'off' will disable the
+    # propagation
+    debug_exceptions = (debug_mode and
+                        options.get('debug_exceptions', 'on') == 'on')
+
+    ignore_errors = configuration_list(options, 'ignore_errors')
+    if ignore_errors:
+        reporter.ignore_errors(ignore_errors)
+    show_errors = configuration_list(options, 'show_errors')
+    if show_errors:
+        reporter.show_errors(show_errors)
+
     return WSGIApplication(
         Zope2.bobo_application,
         Zope2.zpublisher_transactions_manager,
-        not debug_mode,
+        debug_mode,
+        debug_exceptions,
         zope_workers)
+
+# Utilities to read configuration options
+
+def configuration_dict(options, key, sep="."):
+    key_sep = key + sep
+    result = {}
+    for key, value in options.iteritems():
+        if key.startswith(key_sep):
+            prefix, result_key = key.split(sep, 1)
+            result[result_key] = value
+    return result
+
+def configuration_list(options, key, default=''):
+    return filter(lambda value: value,
+                  map(lambda value: value.strip(),
+                      options.get(key, default).split(',')))
+
+def configuration_int(options, key, default='4'):
+    value = options.get(key, default)
+    try:
+        return int(value)
+    except:
+        return int(default)
