@@ -152,7 +152,6 @@ class WSGIPublication(object):
             notify,
             interfaces.PublicationBeforeCommit(self.request))
         self.app.transaction.commit()
-        endInteraction()
         safe_callback(
             self,
             notify,
@@ -166,7 +165,6 @@ class WSGIPublication(object):
             notify,
             interfaces.PublicationBeforeAbort(self.request, None, False))
         self.app.transaction.abort()
-        endInteraction()
         safe_callback(
             self,
             notify,
@@ -198,7 +196,7 @@ class WSGIPublication(object):
             self.finish()
         return data
 
-    def error(self, error, last_known_obj):
+    def render_error(self, error, last_known_obj):
         """Render and log an error.
         """
         if IBrowserPage.providedBy(last_known_obj):
@@ -232,7 +230,7 @@ class WSGIPublication(object):
                 notify(interfaces.PublicationAfterRender(
                         self.request, error_view))
 
-            except Exception as error:
+            except Exception:
                 log_last_error(
                     self.request, self.response, obj=last_known_obj,
                     extra=u"Error while rendering error message")
@@ -243,12 +241,19 @@ class WSGIPublication(object):
             self.response.setStatus(500)
             self.response.setBody(DEFAULT_ERROR_TEMPLATE)
 
+    def should_render_errors(self):
+        """Return True if the error pages should be rendered.
+        """
+        return self.request.environ.get(
+            'wsgi.handleErrors',
+            not self.app.debug_exceptions)
+
     def get_application_root(self):
         """Return a new Zope root for this request.
         """
         # This create the connection to the ZODB, wrap it in the
         # request.  After this, request.close must always be called.
-        root = self.app.application.__bobo_traverse__(self.request)
+        root = self.app.root.__bobo_traverse__(self.request)
         return root.__of__(RequestContainer(REQUEST=self.request))
 
     def get_path_and_method(self, root):
@@ -335,7 +340,7 @@ class WSGIPublication(object):
 
         return self.result()
 
-    def publish_and_manage_errors(self):
+    def publish_and_render_errors(self):
         """Publish the request, manage errors.
         """
         def last_content():
@@ -373,11 +378,11 @@ class WSGIPublication(object):
         except Exception as error:
             content = last_content()
             log_last_error(self.request, self.response, content)
-            if self.response.debug_mode:
-                # If debug mode is on, don't render anything for the error.
+            if not self.should_render_errors():
+                # If we don't handle errors, reraise the exception.
                 raise
 
-            self.error(error, content)
+            self.render_error(error, content)
 
         # Return the result of the response
         return self.result()
@@ -387,7 +392,7 @@ class WSGIPublication(object):
         fails.
         """
         try:
-            data = self.publish_and_manage_errors()
+            data = self.publish_and_render_errors()
         except (ConflictError, Retry):
             self.abort()
             self.publication_done = True
@@ -395,14 +400,15 @@ class WSGIPublication(object):
                 # If can still retry, and didn't send any data yet, do it.
                 logger.info('Conflict, retrying request %s' % (
                         reconstruct_url_from_environ(self.request.environ)))
-                new_request = self.request.retry()
+                endInteraction()
+                request = self.request.retry()
                 try:
-                    new_publication = self.__class__(
-                        self.app, new_request, self.response)
-                    data = new_publication.publish_and_retry()
-                    self.publication_done = new_publication.publication_done
+                    publication = self.__class__(
+                        self.app, request, self.response)
+                    data = publication.publish_and_retry()
+                    self.publication_done = publication.publication_done
                 finally:
-                    new_request.close()
+                    request.close()
             else:
                 # Otherwise, just render a plain error.
                 logger.error('Conflict error for request %s' % (
@@ -423,7 +429,8 @@ class WSGIPublication(object):
             # In case of exception we didn't catch (like
             # AbortPublication), abort all the current transaction.
             self.abort()
-            cleanup()
+            if cleanup is not None:
+                cleanup()
             raise
 
 
@@ -431,12 +438,14 @@ class WSGIApplication(object):
     """Zope WSGI application.
     """
 
-    def __init__(self, application, transaction,
-                 handle_errors=True, concurrency=4):
-        self.application = application
+    def __init__(self, root, transaction,
+                 debug_mode=False, debug_exceptions=True,
+                 concurrency=4):
+        self.root = root
         self.transaction = transaction
         self.memory_maxsize = 2 << 20
-        self.handle_errors = handle_errors
+        self.debug_mode = debug_mode
+        self.debug_exceptions = debug_exceptions
         self.concurrency = threading.Semaphore(concurrency)
 
     def save_input(self, environ):
@@ -474,16 +483,16 @@ class WSGIApplication(object):
         """WSGI entry point.
         """
         try:
-            debug_mode = not environ.get(
-                'wsgi.handleErrors', self.handle_errors)
             self.save_input(environ)
-            response = WSGIResponse(environ, start_response, debug_mode)
+            response = WSGIResponse(environ, start_response, self.debug_mode)
             request = WSGIRequest(environ['wsgi.input'], environ, response)
             publication = WSGIPublication(self, request, response)
 
             self.concurrency.acquire()
             def cleanup(context=None):
                 request.close()
+                endInteraction()
+                noSecurityManager()
                 self.concurrency.release()
 
             return publication(cleanup)
